#ifndef EXAMPLE3_H
#define EXAMPLE3_H

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <string>
#include <algorithm>
#include "ObjectFactory.h"


class Command;


// We'll create our object factory here so it can be used outside of main().
// This would normally be declared as a singleton.
typedef ObjectFactory<Command *(), std::string> CommandFactory;
CommandFactory command_factory;


// We'll store our class identifiers here
static std::string walk_forward_command = "walk_forward";
static std::string fire_weapon_command = "fire_weapon";
static std::string jump_command = "jump";
static std::string help_command = "help";
static std::string exit_command = "exit";


// Our base command class
class Command
{
public:
   virtual ~Command() {}
   virtual void Execute() = 0;
};


// This is our concrete command classes that we'll be creating using the
// object factory.  It really don't do anything other than tell the
// user it is executing
class WalkForwardCommand : public Command
{
public:
   WalkForwardCommand() {}
   virtual ~WalkForwardCommand() {}

   virtual void Execute()
   {
      std::cout << "\nWalking forward...\n" << std::endl;
   }
};

// We can auto-register our WalkForwardCommand class with the object factory
// by adding this line to the top of our WalkForwardCommand.cpp file.
bool walk_forward_registered = command_factory.Register<WalkForwardCommand>(walk_forward_command);



class FireWeaponCommand : public Command
{
public:
   FireWeaponCommand() {}
   virtual ~FireWeaponCommand() {}

   virtual void Execute()
   {
      std::cout << "\nFiring the weapon...\n" << std::endl;
   }
};

// We can auto-register our FireWeaponCommand class with the object factory
// by adding this line to the top of our FireWeaponCommand.cpp file.
bool fire_weapon_registered = command_factory.Register<FireWeaponCommand>(fire_weapon_command);



class JumpCommand : public Command
{
public:
   JumpCommand() {}
   virtual ~JumpCommand() {}

   virtual void Execute()
   {
      std::cout << "\nJumping...\n" << std::endl;
   }
};

// We can auto-register our JumpCommand class with the object factory
// by adding this line to the top of our JumpCommand.cpp file.
bool jump_registered = command_factory.Register<JumpCommand>(jump_command);



class HelpCommand : public Command
{
public:
   HelpCommand() {}
   virtual ~HelpCommand() {}

   virtual void Execute()
   {
      // Let's display a list of all valid commands
      CommandFactory::ConstIterator iter = command_factory.GetBegin();

      while (iter != command_factory.GetEnd())
      {
         std::cout << (*iter).first << std::endl;

         ++iter;
      }

      std::cout << std::endl;
   }
};

// We can auto-register our HelpCommand class with the object factory
// by adding this line to the top of our HelpCommand.cpp file.
bool help_registered = command_factory.Register<HelpCommand>(help_command);



class ExitCommand : public Command
{
public:
   ExitCommand() {}
   virtual ~ExitCommand() {}

   virtual void Execute()
   {
      std::cout << "\nGood bye!\n" << std::endl;

      exit(1);
   }
};

// We can auto-register our ExitCommand class with the object factory
// by adding this line to the top of our ExitCommand.cpp file.
bool exit_registered = command_factory.Register<ExitCommand>(exit_command);


#endif