// main.cpp
//
#include "ObjectFactory.h"
#include "example1.h"
#include "example2.h"


void Example1();
void Example2();


int main(int argc, char* argv[])
{
   //
   // Simply choose which 'example program' you'd like to execute below.
   //
   Example1();
   //Example2();

   return 0;
}

// This example program shows how a programmer can programmatically create
// instances of monsters using the object factory.
//
// Output after running this program is this:
//
// Goblin monster created at coordinate: 34.5, 63, 7.7
// Ogre monster created at coordinate: 83, 14.2, 54.4
//
void Example1()
{
   ObjectFactory3<Monster, float, float, float, int> monster_factory;


   // Register monsters with the object factory class
   monster_factory.Register(OGRE_MONSTER, Type2Type<OgreMonster>());
   monster_factory.Register(GOBLIN_MONSTER, Type2Type<GoblinMonster>());


   // We can instantiated the proper monster class, just by referencing its
   // unique identifier
   Monster *monster1 = monster_factory.Create(GOBLIN_MONSTER, 34.5f, 63.0f, 7.7f);
   Monster *monster2 = monster_factory.Create(OGRE_MONSTER, 83.0f, 14.2f, 54.4f);


   delete monster1;
   delete monster2;
}


// This example program shows how you can use the object factory to create a simple
// console similar to that in Quake.  Users can list all commands registered and
// execute them.  Adding support for new commands is as simple as calling the
// object factory Register method.
//
// Output after running this program can be different each run.
//
void Example2()
{
   std::string command;

   while (1)
   {
      std::cout << "\n"
                << "Welcome to the console!\n"
                << "\n"
                << "Type 'help' for a list of all commands\n"
                << "\n"
                << "? ";
      std::cout.flush();

      std::cin >> command;
      std::cout << "\n";


      // Convert the command string to lowercase
      std::transform(command.begin(), command.end(), command.begin(), tolower);


      // Let's retrieve the command the user wants to execute
      Command *cmd = command_factory.Create(command);


      if (cmd != NULL)
      {
         cmd->Execute();
         delete cmd;
      }
      else
         std::cout << "\nUnknown command!" << std::endl;
   }
}
